#!/bin/bash
cp ../Runs/ef8k65 efdata1
(time ./mc -t 2)      1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k129 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k257 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k513 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k1025 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k2049 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
#cp ../Runs/ef8k4097 efdata1
#./mc -t 2)     1>>apot.swarm 2>>apot.swarm
sleep 2;
cp ../Runs/ef16k65 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k129 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k257 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k513 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k1025 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k2049 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k4097 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
sleep 2
#
cp ../Runs/ef32k65 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k129 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k257 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k513 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k1025 efdata1
(time ./mc -t 2)   1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k2049 efdata1
(time ./mc -t 2)   1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k4097 efdata1
(time ./mc -t 2)   1>>apot.swarm 2>>apot.swarm
#
cp ../Runs/ef8k65 efdata1
(time ./mc -t 4 )      1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k129 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k257 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k513 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k1025 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k2049 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k4097 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
sleep 2;
cp ../Runs/ef16k65 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k129 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k257 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k513 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k1025 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k2049 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k4097 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
sleep 2
cp ../Runs/ef32k65 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k129 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k257 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k513 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k1025 efdata1
(time ./mc -t 4 )   1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k2049 efdata1
(time ./mc -t 4 )   1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k4097 efdata1
(time ./mc -t 4 )   1>>apot.swarm 2>>apot.swarm
#
cp ../Runs/ef8k65 efdata1
(time ./mc -t 2)      1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k129 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k257 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k513 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k1025 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k2049 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k4097 efdata1
(time ./mc -t 2)     1>>apot.swarm 2>>apot.swarm
sleep 2;
cp ../Runs/ef16k65 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k129 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k257 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k513 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k1025 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k2049 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k4097 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
sleep 2
cp ../Runs/ef32k65 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k129 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k257 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k513 efdata1
(time ./mc -t 2)    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k1025 efdata1
(time ./mc -t 2)   1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k2049 efdata1
(time ./mc -t 2)   1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k4097 efdata1
(time ./mc -t 2)   1>>apot.swarm 2>>apot.swarm
#
cp ../Runs/ef8k65 efdata1
(time ./mc -t 4 )      1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k129 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k257 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k513 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k1025 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k2049 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef8k4097 efdata1
(time ./mc -t 4 )     1>>apot.swarm 2>>apot.swarm
sleep 2;
cp ../Runs/ef16k65 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k129 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k257 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k513 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k1025 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k2049 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef16k4097 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
sleep 2
cp ../Runs/ef32k65 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k129 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k257 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k513 efdata1
(time ./mc -t 4 )    1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k1025 efdata1
(time ./mc -t 4 )   1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k2049 efdata1
(time ./mc -t 4 )   1>>apot.swarm 2>>apot.swarm
cp ../Runs/ef32k4097 efdata1
(time ./mc -t 4 )   1>>apot.swarm 2>>apot.swarm
#
